@@ -16,6 +16,7 @@ import (
 	"github.com/docker/docker/api/types/network"
 	"github.com/docker/docker/api/types/volume"
 	"github.com/docker/go-connections/nat"
+	"github.com/docker/go-units"
 	"github.com/docker/swarmkit/agent/exec"
 	"github.com/docker/swarmkit/api"
 	"github.com/docker/swarmkit/api/genericresource"
@@ -209,6 +210,8 @@ func (c *containerConfig) hostConfig() *enginecontainer.HostConfig {
 		PortBindings: c.portBindings(),
 		Init:         c.init(),
 		Isolation:    c.isolation(),
+		CapAdd:       c.spec().CapabilityAdd,
+		CapDrop:      c.spec().CapabilityDrop,
 	}
 
 	// The format of extra hosts on swarmkit is specified in:
@@ -442,7 +445,16 @@ func (c *containerConfig) resources() enginecontainer.Resources {
 	// set pids limit
 	pidsLimit := c.spec().PidsLimit
 	if pidsLimit > 0 {
-		resources.PidsLimit = pidsLimit
+		resources.PidsLimit = &pidsLimit
+	}
+
+	resources.Ulimits = make([]*units.Ulimit, len(c.spec().Ulimits))
+	for i, ulimit := range c.spec().Ulimits {
+		resources.Ulimits[i] = &units.Ulimit{
+			Name: ulimit.Name,
+			Soft: ulimit.Soft,
+			Hard: ulimit.Hard,
+		}
 	}
 
 	// If no limits are specified let the engine use its defaults.
